[CmdletBinding()] 
param() 

# Ignore errors when moule importing
$global:ErrorActionPreference = "SilentlyContinue"
Import-Module -Name "Devart.DbForge.DevOpsAutomation.SqlServer"

if ($null -eq (Get-Module "Devart.DbForge.DevOpsAutomation.SqlServer") ) {

    Write-Host "Installing Devart.DbForge.DevOpsAutomation.SqlServer module..."
    Install-Module -Name Devart.DbForge.DevOpsAutomation.SqlServer -Force
}

Write-Host "Started 'Build Task'...`r`n" 

# Stop on the first error
$global:ErrorActionPreference = 'Stop'

$sourceType = Get-VstsInput -Name 'sourceType'
$targetAuthMethod = Get-VstsInput -Name 'targetAuthMethod'
$targetServer = Get-VstsInput -Name 'targetServer'
$targetDbName = Get-VstsInput -Name 'targetDbName'
$targetConnUserName = Get-VstsInput -Name 'targetConnUserName'
$targetConnPassword = Get-VstsInput -Name 'targetConnPassword'
$additionalParams = Get-VstsInput -Name 'additionalParams'
$sourceScriptFolder = Get-VstsInput -Name 'sourceScriptFolder'
$filterPath = Get-VstsInput -Name 'filterPath'
# The local path on the agent where your source code files are downloaded
if($Env:SYSTEM_HOSTTYPE -eq "build") {
  $agentSourceFolder = $env:SYSTEM_DEFAULTWORKINGDIRECTORY
}
else {
  $agentSourceFolder = Join-Path $env:SYSTEM_DEFAULTWORKINGDIRECTORY $ENV:RELEASE_PRIMARYARTIFACTSOURCEALIAS
}

Write-Host "Initial parameters:"
Write-Host "Database folder = $sourceType"
Write-Host "Subfolder location = $sourceScriptFolder"
Write-Host "Authentication method = $targetAuthMethod"
Write-Host "SQL Server = $targetServer"
Write-Host "Database = $targetDbName"
Write-Host "User Name = $targetConnUserName"
Write-Host "Schema Compare options = $additionalParams"
Write-Host "Schema Compare filter = $filterPath"

Write-Host "`r`n"

# Set script folder
if ($sourceType -eq 'root') {
    $scriptFolder = $agentSourceFolder
}
else {
    $scriptFolder = Join-Path $agentSourceFolder $sourceScriptFolder
}

Write-Host "Full path to the source script folder: $scriptFolder"

Write-Host "`r`n"

# Create database connection
Write-Host "Creating the database connection..."
if ($targetAuthMethod -eq 'sqlServerAuth') {
    $connection = New-DevartSqlDatabaseConnection -Server $targetServer -Database $targetDbName -UserName $targetConnUserName -Password $targetConnPassword
}
else {
    $connection = New-DevartSqlDatabaseConnection -Server $targetServer -Database $targetDbName -WindowsAuthentication $true
}

# Test database connection
Write-Host "Testing the database connection..."
$result = Test-DevartDatabaseConnection -Connection $connection;
if (!$result) {
    Write-Error "Cannot connect to the database"
}

if($filterPath) {
	$filterPath = Join-Path $agentSourceFolder $filterPath
}
Write-Host "Full path to the filter: $filterPath"

# Build database
Write-Host "Building the database..."
$project = Invoke-DevartDatabaseBuild -SourceScriptsFolder $scriptFolder -Connection $connection -SynchronizationOptions $additionalParams -FilterPath $filterPath
if (!$project.Valid) {
    Write-Error "Cannot build the database"
}