[CmdletBinding()] 
param() 

# Ignore errors when moule importing
$global:ErrorActionPreference = "SilentlyContinue"
Import-Module -Name "Devart.DbForge.DevOpsAutomation.SqlServer"

if ($null -eq (Get-Module "Devart.DbForge.DevOpsAutomation.SqlServer") ) {

    Write-Host "Installing Devart.DbForge.DevOpsAutomation.SqlServer module..."
    Install-Module -Name Devart.DbForge.DevOpsAutomation.SqlServer -Force
}

Write-Host "Started 'Document Task'...`r`n" 

# Stop on the first error
$global:ErrorActionPreference = 'Stop'

$targetAuthMethod = Get-VstsInput -Name 'targetAuthMethod'
$targetServer = Get-VstsInput -Name 'targetServer'
$targetDbName = Get-VstsInput -Name 'targetDbName'
$targetConnUserName = Get-VstsInput -Name 'targetConnUserName'
$targetConnPassword = Get-VstsInput -Name 'targetConnPassword'
$additionalOptions = Get-VstsInput -Name 'additionalOptions'
$projectFile = Get-VstsInput -Name 'projectFile'

# The local path on the agent where your source code files are downloaded
if($Env:SYSTEM_HOSTTYPE -eq "build") {
  $agentSourceFolder = $env:SYSTEM_DEFAULTWORKINGDIRECTORY
}
else {
  $agentSourceFolder = Join-Path $env:SYSTEM_DEFAULTWORKINGDIRECTORY $ENV:RELEASE_PRIMARYARTIFACTSOURCEALIAS
}

Write-Host "Initial parameters:"
Write-Host "SQL Server = $targetServer"
Write-Host "Database = $targetDbName"
Write-Host "Authentication method = $targetAuthMethod"
Write-Host "User Name = $targetConnUserName"
Write-Host "AdditionalOptions  = $additionalOptions"
Write-Host "ProjectFile  = $projectFile"

Write-Host "`r`n"

# Create database connection
Write-Host "Creating the database connection..."
if ($targetAuthMethod -eq 'sqlServerAuth') {
    $connection = New-DevartSqlDatabaseConnection -Server $targetServer -Database $targetDbName -UserName $targetConnUserName -Password $targetConnPassword
}
else {
    $connection = New-DevartSqlDatabaseConnection -Server $targetServer -Database $targetDbName -WindowsAuthentication $true
}

# Test database connection
Write-Host "Testing the database connection..."
$result = Test-DevartDatabaseConnection -Connection $connection;
if (!$result) {
    Write-Error "Cannot connect to the database"
}

if ($projectFile) {
    $projectFile = Join-Path $agentSourceFolder $projectFile
}

$output = Join-Path $ENV:AGENT_TempDirectory "Default"

# Documenting database
Write-Host "Documenting the database..."
$result = New-DevartDatabaseDocumentation -Connection $connection -Output $output -AdditionalOptions $AdditionalOptions -Format Pdf -ProjectFile $projectFile
if (!$result) {
    Write-Error "Cannot document the database"
}