[CmdletBinding()] 
param() 

# Ignore errors when moule importing
$global:ErrorActionPreference = "SilentlyContinue"
Import-Module -Name "Devart.DbForge.DevOpsAutomation.SqlServer"

if ($null -eq (Get-Module "Devart.DbForge.DevOpsAutomation.SqlServer") ) {

    Write-Host "Installing Devart.DbForge.DevOpsAutomation.SqlServer module..."
    Install-Module -Name Devart.DbForge.DevOpsAutomation.SqlServer -Force
}

# Stop on the first error
$global:ErrorActionPreference = 'Stop'

Write-Host "Started 'Execute Task'...`r`n" 

$input = Get-VstsInput -Name 'input'
$encoding = Get-VstsInput -Name 'encoding'
$zipPassword = Get-VstsInput -Name 'zipPassword'
$ignoreError = Get-VstsInput -Name 'ignoreError'
$sqlServer = Get-VstsInput -Name 'sqlServer'
$dbName = Get-VstsInput -Name 'dbName'
$authMethod = Get-VstsInput -Name 'authMethod'
$connUserName = Get-VstsInput -Name 'connUserName'
$connPassword = Get-VstsInput -Name 'connPassword'

# The local path on the agent where your source code files are downloaded
if($Env:SYSTEM_HOSTTYPE -eq "build") {
  $agentSourceFolder = $env:SYSTEM_DEFAULTWORKINGDIRECTORY
}
else {
  $agentSourceFolder = Join-Path $env:SYSTEM_DEFAULTWORKINGDIRECTORY $ENV:RELEASE_PRIMARYARTIFACTSOURCEALIAS
}

Write-Host "Initial parameters:"
Write-Host "Input = $input"
Write-Host "Encoding = $encoding"
Write-Host "ZipPassword = $zipPassword"
Write-Host "IgnoreError = $ignoreError"
Write-Host "SQL Server = $sqlServer"
Write-Host "Database = $dbName"
Write-Host "Authentication method = $authMethod"
Write-Host "User Name = $connUserName"

Write-Host "`r`n"

$connection = $null

# Create database connection
Write-Host "Creating the database connection..."
if ($authMethod -eq 'sqlServerAuth') {
    $connection = New-DevartSqlDatabaseConnection -Server $sqlServer -Database $dbName -UserName $connUserName -Password $connPassword
}
else {
    $connection = New-DevartSqlDatabaseConnection -Server $sqlServer -Database $dbName -WindowsAuthentication $true
}

# Test database connection
Write-Host "Testing the database connection..."

$result = Test-DevartDatabaseConnection -Connection $connection;
if (!$result) {
    Write-Error "Cannot connect to the database"
}

Write-Host "`r`n"

# Relative git path
$input = Join-Path $agentSourceFolder $input
Write-Host "Full path to a input: $input"

try {
    $ignoreError = [System.Convert]::ToBoolean($ignoreError)
} 
catch [FormatException] {
    $ignoreError = $false
}

# Execute
Write-Host "Executing..."
if ($ignoreError) {
    $result = Invoke-DevartExecuteScript -Connection $connection -Input $input -Database $dbName -Encoding $encoding -IgnoreError -ZipPassword $zipPassword
}
else {
    $result = Invoke-DevartExecuteScript -Connection $connection -Input $input -Database $dbName -Encoding $encoding -ZipPassword $zipPassword
}
if (!$result) {
    Write-Error "Cannot execute"
}