[CmdletBinding()] 
param() 

# Ignore errors when moule importing
$global:ErrorActionPreference = "SilentlyContinue"
Import-Module -Name "Devart.DbForge.DevOpsAutomation.SqlServer"

if ($null -eq (Get-Module "Devart.DbForge.DevOpsAutomation.SqlServer") ) {

    Write-Host "Installing Devart.DbForge.DevOpsAutomation.SqlServer module..."
    Install-Module -Name Devart.DbForge.DevOpsAutomation.SqlServer -Force
}

# Stop on the first error
$global:ErrorActionPreference = 'Stop'

Write-Host "Started 'Package Task'...`r`n" 

$id = Get-VstsInput -Name 'id'
$sourceType = Get-VstsInput -Name 'sourceType'
$sourceScriptFolder = Get-VstsInput -Name 'sourceScriptFolder'
$includeDocumentation = Get-VstsInput -Name 'includeDocumentation'

# The local path on the agent where your source code files are downloaded
if($Env:SYSTEM_HOSTTYPE -eq "build") {
  $agentSourceFolder = $env:SYSTEM_DEFAULTWORKINGDIRECTORY
}
else {
  $agentSourceFolder = Join-Path $env:SYSTEM_DEFAULTWORKINGDIRECTORY $ENV:RELEASE_PRIMARYARTIFACTSOURCEALIAS
}

$documentationPath = ""

Write-Host "Initial parameters:"
Write-Host "ID = $id"
Write-Host "Include database documentation = $includeDocumentation"
Write-Host "Database folder = $sourceType"
Write-Host "Subfolder location = $sourceScriptFolder"

Write-Host "`r`n"

Write-Host "Start creating a NuGet package file"

# Set script folder
if ($sourceType -eq 'root') {
    $scriptFolder = $agentSourceFolder
}
else {

    if (!$sourceScriptFolder) {
        Write-Error "No source directory is set up"
    }

    if ([System.IO.Path]::IsPathRooted($sourceScriptFolder) -eq $False) {
        $scriptFolder = Join-Path $agentSourceFolder $sourceScriptFolder
    }
    else {
        $scriptFolder = $sourceScriptFolder
    }
}

Write-Host "Source script folder: $scriptFolder"

$fileName = $id

$ext = [System.IO.Path]::GetExtension($fileName)
if ($ext -eq ".DRAFT" ) {
    $fileName = $fileName -replace $ext
}

$versionPattern = '[0-9]+\.[0-9]+\.[0-9]+(?:\.[0-9]+)?';

# Set version
$version = [Regex]::Matches($fileName, $versionPattern).Value
if (!$version) {

    $versionErrorMessage = "A version is not specified or is incorrect in a file name. The file name must contain a version number"
    Write-Error $versionErrorMessage
}
Write-Output "Version: $version"

# Set outputId
$outputId = $fileName -replace $version -replace "\.$"
Write-Output "Output Id: $outputId"

$filePath = Join-Path $agentSourceFolder $id
$filePath += ".nupkg"
if (Test-Path -Path $filePath) {
    Write-Error "A file with the '$id' already exists"
}

Write-Host "Creating a project..."
# Create Project
$project = New-DevartDatabaseProject -SourceScriptsFolder $scriptFolder

# set up documentation path
try {
    $includeDocumentation = [System.Convert]::ToBoolean($includeDocumentation)
} 
catch [FormatException] {
    $includeDocumentation = $false
}

if ($includeDocumentation) {
    $documentationPath = Join-Path $ENV:AGENT_TempDirectory "Default.pdf"  
    if (!(Test-Path $documentationPath)) {
        $documentationPath = ""
        Write-Warning "The documentation does not exist"       
    }
}

Write-Host "Setting up a package info..."
# Create PackageInfo
Set-DevartPackageInfo -Id $outputId -Project $project -Documentation $documentationPath -Version $version

Write-Output "Output directory: $agentSourceFolder"

Write-Host "Exporting a database project..."
# Create package
$result = Export-DevartDatabaseProject -Project $project -OutputDirectory $agentSourceFolder -Format NuGet -Overwrite

if (!$result) {
    Write-Error "Cannot create a package in the folder  $agentSourceFolder"
}