[CmdletBinding()] 
param() 

# Ignore errors when moule importing
$global:ErrorActionPreference = "SilentlyContinue"
Import-Module -Name "Devart.DbForge.DevOpsAutomation.SqlServer"

if ($null -eq (Get-Module "Devart.DbForge.DevOpsAutomation.SqlServer") ) {

    Write-Host "Installing Devart.DbForge.DevOpsAutomation.SqlServer module..."
    Install-Module -Name Devart.DbForge.DevOpsAutomation.SqlServer -Force
}

# Stop on the first error
$global:ErrorActionPreference = 'Stop'

Write-Host "Started 'Publish Task'...`r`n" 

$id = Get-VstsInput -Name 'id'
$packageFolderType = Get-VstsInput -Name 'packageFolderType'
$packageFolder = Get-VstsInput -Name 'packageFolder'
$uRLOrPackagesSourceKey = Get-VstsInput -Name 'uRLOrPackagesSourceKey'
$aPIKey = Get-VstsInput -Name 'aPIKey'

# The local path on the agent where your source code files are downloaded
if($Env:SYSTEM_HOSTTYPE -eq "build") {
  $agentSourceFolder = $env:SYSTEM_DEFAULTWORKINGDIRECTORY
}
else {
  $agentSourceFolder = Join-Path $env:SYSTEM_DEFAULTWORKINGDIRECTORY $ENV:RELEASE_PRIMARYARTIFACTSOURCEALIAS
}

Write-Host "Initial parameters:"
Write-Host "ID = $id"
Write-Host "Package folder = $packageFolderType"
Write-Host "Subfolder location = $packageFolder"
Write-Host "URL or packages source key = $uRLOrPackagesSourceKey"
Write-Host "API key = $aPIKey"

Write-Host "`r`n"

# Set script folder
if ($packageFolderType -eq 'root') {
    $packageFolder = $agentSourceFolder
    $packageFullFilePath = Join-Path $packageFolder $id
}
else {

    if (!$packageFolder) {
        Write-Error "No package folder is set up"
    }

    if ([System.IO.Path]::IsPathRooted($packageFolder) -eq $False) {
        $packageFolder = Join-Path $agentSourceFolder $packageFolder
        $packageFullFilePath = Join-Path $packageFolder $id
    }
    else{
        $packageFullFilePath = $packageFolder
    }
}

Write-Host "Package folder: $packageFolder"
Write-Host "Full path to package file: $packageFullFilePath"

Write-Host "Publishing  a package... : $packageFolder"
$result = Publish-DevartDatabaseProject -Project $packageFullFilePath -Repository $uRLOrPackagesSourceKey -ApiKey $aPIKey

if (!$result) {
    Write-Error "Cannot publish a package $packageFullFilePath"
}
