[CmdletBinding()] 
param() 

# Ignore errors when moule importing
$global:ErrorActionPreference = "SilentlyContinue"
Import-Module -Name "Devart.DbForge.DevOpsAutomation.SqlServer"

if ($null -eq (Get-Module "Devart.DbForge.DevOpsAutomation.SqlServer") ) {

    Write-Host "Installing Devart.DbForge.DevOpsAutomation.SqlServer module..."
    Install-Module -Name Devart.DbForge.DevOpsAutomation.SqlServer -Force
}

Write-Host "Started 'Sync Task'...`r`n" 

# Stop on the first error
$global:ErrorActionPreference = 'Stop'

$sourceLocalNugetPath = Get-VstsInput -Name 'sourceLocalNugetPath'
$sourceType = Get-VstsInput -Name 'sourceType'
$sourceAuthMethod = Get-VstsInput -Name 'sourceAuthMethod'
$sourceServer = Get-VstsInput -Name 'sourceServer'
$sourceDbName = Get-VstsInput -Name 'sourceDbName'
$sourceConnUserName = Get-VstsInput -Name 'sourceConnUserName'
$sourceConnPassword = Get-VstsInput -Name 'sourceConnPassword'
$targetAuthMethod = Get-VstsInput -Name 'targetAuthMethod'
$targetServer = Get-VstsInput -Name 'targetServer'
$targetDbName = Get-VstsInput -Name 'targetDbName'
$targetConnUserName = Get-VstsInput -Name 'targetConnUserName'
$targetConnPassword = Get-VstsInput -Name 'targetConnPassword'
$additionalParams = Get-VstsInput -Name 'additionalParams'
$filterPath = Get-VstsInput -Name 'filterPath'

# The local path on the agent where your source code files are downloaded
if($Env:SYSTEM_HOSTTYPE -eq "build") {
  $agentSourceFolder = $env:SYSTEM_DEFAULTWORKINGDIRECTORY
}
else {
  $agentSourceFolder = Join-Path $env:SYSTEM_DEFAULTWORKINGDIRECTORY $ENV:RELEASE_PRIMARYARTIFACTSOURCEALIAS
}

Write-Host "Initial parameters:"
Write-Host "Source = $sourceType"
Write-Host "Name = $sourceLocalNugetPath"
Write-Host "Authentication method = $sourceAuthMethod"
Write-Host "Source SQL Server = $sourceServer"
Write-Host "Source Database = $sourceDbName"
Write-Host "Source User Name = $sourceConnUserName"
Write-Host "Authentication method = $targetAuthMethod"
Write-Host "Target SQL Server = $targetServer"
Write-Host "Target Database = $targetDbName"
Write-Host "Target User Name = $targetConnUserName"
Write-Host "Schema Compare options = $additionalParams"
Write-Host "Schema Compare filter = $filterPath"

Write-Host "`r`n"

$targetConnection = $null

# Create target database connection
Write-Host "Creating the target database connection..."
if ($targetAuthMethod -eq 'sqlServerAuth') {
    $targetConnection = New-DevartSqlDatabaseConnection -Server $targetServer -Database $targetDbName -UserName $targetConnUserName -Password $targetConnPassword
}
else {
    $targetConnection = New-DevartSqlDatabaseConnection -Server $targetServer -Database $targetDbName -WindowsAuthentication $true
}

# Test target database connection
Write-Host "Testing the target database connection..."
$result = Test-DevartDatabaseConnection -Connection $targetConnection;
if (!$result) {
    Write-Error "Cannot connect to the target database"
}

if ($filterPath) {
    $filterPath = Join-Path $agentSourceFolder $filterPath
}
Write-Host "Full path to the filter: $filterPath"
	
if ($sourceType -eq 'database') {

    Write-Host "Starting database synchronization..."    

    $sourceConnection = $null

    # Create source database connection
    Write-Host "Creating the source database connection..."
    if ($sourceAuthMethod -eq 'sqlServerAuth') {
        $sourceConnection = New-DevartSqlDatabaseConnection -Server $sourceServer -Database $sourceDbName -UserName $sourceConnUserName -Password $sourceConnPassword
    }
    else {
        $sourceConnection = New-DevartSqlDatabaseConnection -Server $sourceServer -Database $sourceDbName -WindowsAuthentication $true
    }

    # Test source database connection
    Write-Host "Testing the source database connection..."
    $result = Test-DevartDatabaseConnection -Connection $sourceConnection;
    if (!$result) {
        Write-Error "Cannot connect to the source database"
    }

    Write-Host "`r`n"

    # Sync databases
    Write-Host "Synchronizing databases..."
    $result = Invoke-DevartSyncDatabaseSchema -Source $sourceConnection -Target $targetConnection -SynchronizationOptions $additionalParams -FilterPath $filterPath
    if (!$result) {
        Write-Error "Cannot synchronize the databases"
    }
}
else {

    Write-Host "Starting synchronization using a database package..."    
    $sourceLocalNugetPath = Join-Path $agentSourceFolder $sourceLocalNugetPath
    Write-Host "Full path to a package: $sourceLocalNugetPath"

    # Sync databases
    Write-Host "Synchronizing a database using a package..."
    $result = Invoke-DevartSyncDatabaseSchema -Source $sourceLocalNugetPath -Target $targetConnection -SynchronizationOptions $additionalParams -FilterPath $filterPath
    if (!$result) {
        Write-Error "Cannot synchronize a database using a package..."
    }
}