[CmdletBinding()] 
param() 

# Ignore errors when moule importing
$global:ErrorActionPreference = "SilentlyContinue"
Import-Module -Name "Devart.DbForge.DevOpsAutomation.SqlServer"

if ($null -eq (Get-Module "Devart.DbForge.DevOpsAutomation.SqlServer")) {
   
    Install-Module -Name Devart.DbForge.DevOpsAutomation.SqlServer -Force
}

Write-Host "Started 'Test Task'...`r`n" 

# Stop on the first error
$global:ErrorActionPreference = 'Stop'

$targetAuthMethod = Get-VstsInput -Name 'targetAuthMethod'
$targetServer = Get-VstsInput -Name 'targetServer'
$targetDbName = Get-VstsInput -Name 'targetDbName'
$targetConnUserName = Get-VstsInput -Name 'targetConnUserName'
$targetConnPassword = Get-VstsInput -Name 'targetConnPassword'
$testsType = Get-VstsInput -Name 'testsType'
$selectedTests = Get-VstsInput -Name 'selectedTests'

Write-Host "Initial parameters:"
Write-Host "Authentication method = $targetAuthMethod"
Write-Host "SQL Server = $targetServer"
Write-Host "Database = $targetDbName"
Write-Host "User Name = $targetConnUserName"
Write-Host "Run tests = $testsType"
Write-Host "Tests to be run = $selectedTests"

Write-Host "`r`n"

# Set unit tests
$unittests = $null
if ($testsType -eq 'selectedOnly') {
    $unittests = $selectedTests
}

# Create database connection
Write-Host "Creating the database connection..."
if ($targetAuthMethod -eq 'sqlServerAuth') {
    $connection = New-DevartSqlDatabaseConnection -Server $targetServer -Database $targetDbName -UserName $targetConnUserName -Password $targetConnPassword
}
else {
    $connection = New-DevartSqlDatabaseConnection -Server $targetServer -Database $targetDbName -WindowsAuthentication $true
}

# Test database connection
Write-Host "Testing the database connection..."
$result = Test-DevartDatabaseConnection -Connection $connection;
if (!$result) {
    Write-Error "Cannot connect to the database"
}

# Run unit tests
Write-Host "Running unit tests..."
$result = Invoke-DevartDatabaseTests -InputObject $connection -InstalltSQLtFramework -UnitTests $unittests
if (!$result) {
    Write-Error "Unit tests have failed"
}